//---------------------------------------------------------------------------
//Header Includes
//---------------------------------------------------------------------------
#ifndef ModelManager_H
#define ModelManager_H
#include <xmlparse.h>

//---------------------------------------------------------------------------
//Constants
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Class definition
//---------------------------------------------------------------------------

/**
   The model manager is a central part of the system
   since it is the one that actually stores the DOM
   representation of the document(
   an aggregation of xmlNodes). The Role of the
   Model manager is to be the listener of command
   requests (further delegated to the
   CommandManager). ModelManager is a singleton.

*/
class ModelManager {
  public:
  /**@name Get Methods
   */
  //@{
    /// Get the actual DOM representation of the document
    const xmlNode *getDOM();
  //@}

  /**@name View interaction methods
  * This Methods are needed for the interaction with the views
  */
  //@{
   /** 
      subscribeView is called whenever a view wants to register in
      order to listen to notifications (and be able to do well behaved
      modifications)
      @param view the view that will listen to notifications
   */
   void subscribeView(AbstractView *view); 

     /** 
      This method is called whenever a view wants to send a command for execution.
      The method is provided here in order to decouple the views from the command 
      manager.
      @param c the command to be executed
      @exception SpaceExceeded if the space available for waiting commands is 
                 exceeded
   */
   void execute(AbstractCommand *c);

     /** 
      This method is called when a view requests an undo.
      If the undo is successful (i.e. if there is no NoFurtherUndo exception thrown),
      a notification (deep or shallow, depending on the command undoed) is issued
      to every registered view. Note that after an undo, the last command is not the undo
      itself, but the command previous to the undoed.
      @exception NoFurtherUndo if there is no commands left to undo
      @see notifyAll
   */
   void undo();

     /** 
      This method is called when a view requests a redo.
      If the redo is successful (i.e. if there is no NoFurtherRedo exception thrown),
      a notification (deep or shallow, depending on the command redoed) is issued
      to every registered view. Note that after a redo, the current command is still the
      current itself, but the command previous to the redoed.
      @see notifyAll
   */
   void redo();
  //@}
 
  protected:
  /**@name Notification methods
  * This Methods are needed for the interaction with the views
  */
  //@{
   /** used to send all views Notification events.
       Called by the Command Manager (*not* the views) in order to notify the views
       of changes inflicted in the model by the commands.
       @param n the notification object to be sent.
    */
   void notifyAll(Notification *n);
  //@}
  private:
    /// The whole document
    xmlNode *root;
    /// The current node
    xmlNode *current;
    /// Registered Views
    list<AbstractView *> views;
};
#endif


